@@ -1,4 +1,5 @@
-//! Updates dispatching.
+//! Old updates dispatching (**DEPRECATED**: use [`crate::dispatching2`]
+//! instead).
 //!
 //! The key type here is [`Dispatcher`]. It encapsulates [`Bot`] and handlers
 //! for [all the update kinds].
@@ -45,11 +46,12 @@
 //! [`tokio::sync::mpsc::UnboundedReceiver`]: https://docs.rs/tokio/0.2.11/tokio/sync/mpsc/struct.UnboundedReceiver.html
 //! [examples/dialogue_bot]: https://github.com/teloxide/teloxide/tree/master/examples/dialogue_bot
 
+#![allow(deprecated)]
+
 pub mod dialogue;
 pub mod stop_token;
 pub mod update_listeners;
 
-#[cfg(feature = "ctrlc_handler")]
 pub(crate) mod repls;
 
 mod dispatcher;
@@ -57,7 +59,8 @@ mod dispatcher_handler;
 mod dispatcher_handler_rx_ext;
 mod update_with_cx;
 
-pub use dispatcher::{Dispatcher, IdleShutdownError, ShutdownToken};
+pub use crate::utils::shutdown_token::{IdleShutdownError, ShutdownToken};
+pub use dispatcher::Dispatcher;
 pub use dispatcher_handler::DispatcherHandler;
 pub use dispatcher_handler_rx_ext::DispatcherHandlerRxExt;
 use tokio::sync::mpsc::UnboundedReceiver;
@@ -66,4 +69,5 @@ pub use update_with_cx::{UpdateWithCx, UpdateWithCxRequesterType};
 /// A type of a stream, consumed by [`Dispatcher`]'s handlers.
 ///
 /// [`Dispatcher`]: crate::dispatching::Dispatcher
+#[deprecated(note = "Use dispatching2 instead")]
 pub type DispatcherHandlerRx<R, Upd> = UnboundedReceiver<UpdateWithCx<R, Upd>>;
